/* NAME:
 *	uclnt - manage the UNFS client data base
 *
 * DESCRIPTION:
 *	These routines simply keep track of which port is used to talk
 *	to the server for each mount point.  If this info could be
 *	gleaned from a SunOS kernel it would be cool.
 *
 *
 * AUTHOR:
 *	Simon J. Gerraty <sjg@zen.void.oz.au>
 */
/*
 *      @(#)Copyright (c) 1994, Simon J. Gerraty.
 *      
 *      This is free software.  It comes with NO WARRANTY.
 *      Permission to use, modify and distribute this source code 
 *      is granted subject to the following conditions.
 *      1/ that the above copyright notice and this notice 
 *      are preserved in all copies and that due credit be given 
 *      to the author.  
 *      2/ that any changes to this code are clearly commented 
 *      as such so that the author does not get blamed for bugs 
 *      other than his own.
 *      
 *      Please send copies of changes and bug-fixes to:
 *      sjg@zen.void.oz.au
 */

#ifndef lint
static char RCSid[] = "$Id: uclnt.c,v 1.4 1995/05/25 11:54:51 sjg Exp $";
#endif
#include <stdio.h>
#include <sys/param.h>
#include <sys/mount.h>
#include <sys/socket.h>
#include <sys/socketvar.h>
#include <sys/stat.h>
#include <sys/syslog.h>
#include <string.h>

#include "pathnames.h"

#ifdef MAIN
#undef _PATH_UCLNTS
#define _PATH_UCLNTS "/tmp/uclnts"
#endif

static char LckFile[256];

FILE *
uclnts_open(mode)
	char *mode;
{
	FILE *fh;
	int i;

	if ((fh = fopen(_PATH_UCLNTS, mode)) == NULL) {
		syslog(LOG_ERR, "%s: %m", _PATH_UCLNTS);
		return NULL;
	}
	sprintf(LckFile, "%s.lck", _PATH_UCLNTS);
	for (i = 0; i < 5; i++) {
		if (link(_PATH_UCLNTS, LckFile) == 0)
			return fh;
		else
			sleep(1);		/* shouldn't take long */
	}
	fclose(fh);
	return NULL;
}

void
uclnts_close(fh)
	FILE *fh;
{
	if (fh) {
		fclose(fh);
		unlink(LckFile);
	}
}

int
uclnts_add(spec, mntpt, port)
	char *spec, *mntpt;
	int port;
{
	FILE *fh;

	if ((fh = uclnts_open("a")) != NULL) {
		(void) fprintf(fh, "%s,%s\t%d\t=%d\n",
			       spec, mntpt, port, getpid());
		uclnts_close(fh);
	}
	return (fh != NULL);
}

int
uclnts_get2(spec, mntpt, pidp)
	char *spec, *mntpt;
	int *pidp;
{
	FILE *fh;
	char buf[512];
	char want[512];
	int len, port = -1;

	
	if ((fh = uclnts_open("r")) != NULL) {
		(void) sprintf(want, "%s,%s\t", spec, mntpt);
		len = strlen(want);
		
		while (fgets(buf, sizeof (buf), fh) != NULL) {
			if (strncmp(buf, want, len) == 0) {
				port = atoi(&buf[len]);
				if (pidp) {
					register char *rcp;

					if ((rcp = strrchr(buf, '=')) != NULL)
						*pidp = atoi(++rcp);
					else
						*pidp = 0;
				}
				break;
			}
		}
		uclnts_close(fh);
	}
	return (port);
}

int
uclnts_get(spec, mntpt)
	char *spec, *mntpt;
{
	return uclnts_get2(spec, mntpt, (int *)0);
}

int
uclnts_delete(spec, mntpt)
	char *spec, *mntpt;
{
	FILE *th, *fh;
	char tmpf[256];
	char buf[512];
	char want[512];
	int len;
	
	if ((fh = uclnts_open("r")) != NULL) {
		(void) sprintf(tmpf, "%s.XXXXXX", _PATH_UCLNTS);
		(void) mktemp(tmpf);

		if ((th = fopen(tmpf, "w")) == NULL) {
			syslog(LOG_ERR, "%s: %m", tmpf);
			uclnts_close(fh);
			return -1;
		}
		(void) sprintf(want, "%s,%s\t", spec, mntpt);
		len = strlen(want);
		while (fgets(buf, sizeof (buf), fh) != NULL) {
			if (strncmp(buf, want, len) == 0) {
				continue;	/* skip it */
			}
			(void) fprintf(th, "%s", buf); /* avoid '%' in buf */
		}
		fclose(th);
		fclose(fh);
		rename(tmpf, _PATH_UCLNTS);
		uclnts_close(fh);
	}
	return 0;
}


#ifdef MAIN
int
main(argc, argv)
	int argc;
	char **argv;
{
	int port, pid;
	
	unlink(_PATH_UCLNTS);
	openlog("uclnts", LOG_CONS, LOG_LOCAL1);
	if (uclnts_add("localhost:/tmp", "/test/tmp", 8192)) {
		uclnts_add("localhost:/tmp2", "/test/tmp2", 8193);

		(void) printf("port for /test/tmp is %d\n",
			      uclnts_get("localhost:/tmp", "/test/tmp"));
		port = uclnts_get2("localhost:/tmp2", "/test/tmp2", &pid);
		
		(void) printf("port for /test/tmp2 is %d, pid is %d (%d)\n",
			      port, pid, getpid());
		sleep(5);
		
		uclnts_delete("localhost:/tmp2", "/test/tmp2");
	} else {
		(void) fprintf(stderr, "cannot lock '%s'\n",
			       _PATH_UCLNTS);
	}
	exit(0);
}
#endif
